@@ -6,14 +6,16 @@ use warnings;
 
 BEGIN {
 	$Error::TypeTiny::AUTHORITY = 'cpan:TOBYINK';
-	$Error::TypeTiny::VERSION   = '1.004002';
+	$Error::TypeTiny::VERSION   = '1.010006';
 }
 
-use overload
+$Error::TypeTiny::VERSION =~ tr/_//d;
+
+require Type::Tiny;
+__PACKAGE__->Type::Tiny::_install_overloads(
 	q[""]    => sub { $_[0]->to_string },
 	q[bool]  => sub { 1 },
-	fallback => 1,
-;
+);
 
 our %CarpInternal;
 $CarpInternal{$_}++ for qw(
@@ -290,7 +292,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2018 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
