@@ -6,29 +6,28 @@ use warnings;
 
 BEGIN {
 	$Type::Coercion::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Coercion::VERSION   = '1.002002';
+	$Type::Coercion::VERSION   = '1.010006';
 }
 
+$Type::Coercion::VERSION =~ tr/_//d;
+
 use Eval::TypeTiny qw<>;
 use Scalar::Util qw< blessed >;
 use Types::TypeTiny qw<>;
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-use overload
-	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? overload::StrVal($_[0]) : $_[0]->display_name },
+require Type::Tiny;
+
+__PACKAGE__->Type::Tiny::_install_overloads(
+	q("")      => sub { caller =~ m{^(Moo::HandleMoose|Sub::Quote)} ? $_[0]->_stringify_no_magic : $_[0]->display_name },
 	q(bool)    => sub { 1 },
 	q(&{})     => "_overload_coderef",
-	fallback   => 1,
-;
+);
 
-BEGIN {
-	require Type::Tiny;
-	overload->import(
-		q(~~)    => sub { $_[0]->has_coercion_for_value($_[1]) },
-		fallback => 1, # 5.10 loses the fallback otherwise
-	) if Type::Tiny::SUPPORT_SMARTMATCH();
-}
+__PACKAGE__->Type::Tiny::_install_overloads(
+	q(~~)    => sub { $_[0]->has_coercion_for_value($_[1]) },
+) if Type::Tiny::SUPPORT_SMARTMATCH();
 
 sub _overload_coderef
 {
@@ -41,7 +40,8 @@ sub _overload_coderef
 	}
 	else
 	{
-		$self->{_overload_coderef} ||= sub { $self->coerce(@_) };
+		Scalar::Util::weaken(my $weak = $self);
+		$self->{_overload_coderef} ||= sub { $weak->coerce(@_) };
 	}
 	
 	$self->{_overload_coderef};
@@ -73,6 +73,10 @@ sub new
 	return $self;
 }
 
+sub _stringify_no_magic {
+	sprintf('%s=%s(0x%08x)', blessed($_[0]), Scalar::Util::reftype($_[0]), Scalar::Util::refaddr($_[0]));
+}
+
 sub name                   { $_[0]{name} }
 sub display_name           { $_[0]{display_name}      ||= $_[0]->_build_display_name }
 sub library                { $_[0]{library} }
@@ -104,7 +108,7 @@ sub _maybe_restore_type_constraint
 	{
 		return Type::Tiny->new(constraint => $check);
 	}
-	return;
+	return; # uncoverable statement
 }
 
 sub add
@@ -229,15 +233,21 @@ sub add_type_coercions
 	
 	while (@args)
 	{
-		my $type     = Types::TypeTiny::to_TypeTiny(shift @args);
-		my $coercion = shift @args;
+		my $type = Types::TypeTiny::to_TypeTiny(shift @args);
 		
-		_croak "Types must be blessed Type::Tiny objects"
-			unless Types::TypeTiny::TypeTiny->check($type);
-		_croak "Coercions must be code references or strings"
-			unless Types::TypeTiny::StringLike->check($coercion) || Types::TypeTiny::CodeLike->check($coercion);
-		
-		push @{$self->type_coercion_map}, $type, $coercion;
+		if (blessed $type and my $method = $type->can('type_coercion_map'))
+		{
+			push @{$self->type_coercion_map}, @{$method->($type)};
+		}
+		else
+		{
+			my $coercion = shift @args;
+			_croak "Types must be blessed Type::Tiny objects"
+				unless Types::TypeTiny::TypeTiny->check($type);
+			_croak "Coercions must be code references or strings"
+				unless Types::TypeTiny::StringLike->check($coercion) || Types::TypeTiny::CodeLike->check($coercion);
+			push @{$self->type_coercion_map}, $type, $coercion;
+		}
 	}
 	
 	$self->_clear_compiled_coercion;
@@ -871,7 +881,7 @@ to call C<add>. Support for this was dropped after 0.040.
 
 =over
 
-=item B<< Attempt to add coercion code to a Type::Coercion which has been frozen >>
+=item I<< Attempt to add coercion code to a Type::Coercion which has been frozen >>
 
 Type::Tiny type constraints are designed as immutable objects. Once you've
 created a constraint, rather than modifying it you generally create child
@@ -926,7 +936,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
