@@ -11,7 +11,7 @@ BEGIN {
 
 BEGIN {
 	$Type::Tiny::AUTHORITY   = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION     = '1.010002';
+	$Type::Tiny::VERSION     = '1.010006';
 	$Type::Tiny::XS_VERSION  = '0.016';
 }
 
@@ -1300,11 +1300,11 @@ sub can
 			my $method = $self->_lookup_my_method($1);
 			return $method if $method;
 		}
-	}
-
-	if ($self->{is_object} && $object_methods{$_[0]}) {
-		require Type::Tiny::ConstrainedObject;
-		return Type::Tiny::ConstrainedObject->can($_[0]);
+		if ($self->{is_object} && $object_methods{$_[0]})
+		{
+			require Type::Tiny::ConstrainedObject;
+			return Type::Tiny::ConstrainedObject->can($_[0]);
+		}
 	}
 	
 	return;
@@ -1328,13 +1328,13 @@ sub AUTOLOAD
 			my $method = $self->_lookup_my_method($1);
 			return &$method($self, @_) if $method;
 		}
-	}
-	
-	if ($self->{is_object} && $object_methods{$m}) {
-		require Type::Tiny::ConstrainedObject;
-		unshift @_, $self;
-		no strict 'refs';
-		goto \&{"Type::Tiny::ConstrainedObject::$m"};
+		if ($self->{is_object} && $object_methods{$m})
+		{
+			require Type::Tiny::ConstrainedObject;
+			unshift @_, $self;
+			no strict 'refs';
+			goto \&{"Type::Tiny::ConstrainedObject::$m"};
+		}
 	}
 	
 	_croak q[Can't locate object method "%s" via package "%s"], $m, ref($self)||$self;
@@ -2204,7 +2204,7 @@ L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny>.
 
 =head1 SEE ALSO
 
-L<The Type::Tiny homepage|http://typetiny.toby.ink/>.
+L<The Type::Tiny homepage|https://typetiny.toby.ink/>.
 
 L<Type::Tiny::Manual>, L<Type::API>.
 
