@@ -4,26 +4,49 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '1.002002';
+our $VERSION   = '1.010006';
+
+$VERSION =~ tr/_//d;
 
 use Scalar::Util qw< blessed refaddr weaken >;
 
-our @EXPORT_OK = ( __PACKAGE__->type_names, qw/to_TypeTiny/ );
+our @EXPORT_OK = (
+	map(@{[ $_, "is_$_", "assert_$_" ]}, __PACKAGE__->type_names),
+	qw/to_TypeTiny/
+);
+our %EXPORT_TAGS = (
+	types     => [                  __PACKAGE__->type_names ],
+	is        => [ map "is_$_",     __PACKAGE__->type_names ],
+	assert    => [ map "assert_$_", __PACKAGE__->type_names ],
+);
 
 my %cache;
 
+# This `import` method is designed to avoid loading Exporter::Tiny.
+# This is so that if you stick to only using the purely OO parts of
+# Type::Tiny, you can skip loading the exporter.
+#
 sub import
 {
-	# do the shuffle!
-	no warnings "redefine";
-	our @ISA = qw( Exporter::Tiny );
-	require Exporter::Tiny;
-	my $next = \&Exporter::Tiny::import;
-	*import = $next;
-	my $class = shift;
-	my $opts  = { ref($_[0]) ? %{+shift} : () };
-	$opts->{into} ||= scalar(caller);
-	return $class->$next($opts, @_);
+	# If this sub succeeds, it will replace itself.
+	# uncoverable subroutine
+	return unless @_ > 1;                         # uncoverable statement
+	no warnings "redefine";                       # uncoverable statement
+	our @ISA = qw( Exporter::Tiny );              # uncoverable statement
+	require Exporter::Tiny;                       # uncoverable statement
+	my $next = \&Exporter::Tiny::import;          # uncoverable statement
+	*import = $next;                              # uncoverable statement
+	my $class = shift;                            # uncoverable statement
+	my $opts  = { ref($_[0]) ? %{+shift} : () };  # uncoverable statement
+	$opts->{into} ||= scalar(caller);             # uncoverable statement
+	return $class->$next($opts, @_);              # uncoverable statement
+}
+
+for (__PACKAGE__->type_names) {
+	eval qq{
+		sub is_$_     { $_()->check(shift) }
+		sub assert_$_ { $_()->assert_return(shift) }
+	};
 }
 
 sub meta
@@ -33,7 +56,7 @@ sub meta
 
 sub type_names
 {
-	qw( CodeLike StringLike TypeTiny HashLike ArrayLike );
+	qw( CodeLike StringLike TypeTiny HashLike ArrayLike _ForeignTypeConstraint );
 }
 
 sub has_type
@@ -69,13 +92,58 @@ sub get_coercion
 	&{$_[0]}();  # uncoverable statement
 }
 
+my ($__get_linear_isa_dfs, $tried_mro);
+$__get_linear_isa_dfs = sub {
+	if (!$tried_mro && eval { require mro }) {
+		$__get_linear_isa_dfs = \&mro::get_linear_isa;
+		goto $__get_linear_isa_dfs;
+	}
+	no strict 'refs';
+	my $classname = shift;
+	my @lin = ($classname);
+	my %stored;
+	foreach my $parent (@{"$classname\::ISA"})
+	{
+		my $plin = $__get_linear_isa_dfs->($parent);
+		foreach (@$plin) {
+			next if exists $stored{$_};
+			push(@lin, $_);
+			$stored{$_} = 1;
+		}
+	}
+	return \@lin;
+};
+
+sub _check_overload
+{
+	my $package = shift;
+	if (ref $package) {
+		$package = blessed($package);
+		return !!0 if !defined $package;
+	}
+	my $op  = shift;
+	my $mro = $__get_linear_isa_dfs->($package);
+	foreach my $p (@$mro) {
+		my $fqmeth = $p . q{::(} . $op;
+		return !!1 if defined &{$fqmeth};
+	}
+	!!0;
+}
+
+sub _get_check_overload_sub {
+	if ($Type::Tiny::AvoidCallbacks) {
+		return '(sub { require overload; overload::Overloaded(ref $_[0] or $_[0]) and overload::Method((ref $_[0] or $_[0]), $_[1]) })->'
+	}
+	return 'Types::TypeTiny::_check_overload';
+}
+
 sub StringLike ()
 {
 	require Type::Tiny;
 	$cache{StringLike} ||= "Type::Tiny"->new(
 		name       => "StringLike",
-		constraint => sub {    defined($_   ) && !ref($_   ) or Scalar::Util::blessed($_   ) && overload::Method($_   , q[""])  },
-		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[""])/ },
+		constraint => sub {    defined($_   ) && !ref($_   ) or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[""])  },
+		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[""])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -85,8 +153,8 @@ sub HashLike ()
 	require Type::Tiny;
 	$cache{HashLike} ||= "Type::Tiny"->new(
 		name       => "HashLike",
-		constraint => sub {    ref($_   ) eq q[HASH] or Scalar::Util::blessed($_   ) && overload::Method($_   , q[%{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[\%{}])/ },
+		constraint => sub {    ref($_   ) eq q[HASH] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[%{}])  },
+		inlined    => sub { qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\%{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -96,8 +164,8 @@ sub ArrayLike ()
 	require Type::Tiny;
 	$cache{ArrayLike} ||= "Type::Tiny"->new(
 		name       => "ArrayLike",
-		constraint => sub {    ref($_   ) eq q[ARRAY] or Scalar::Util::blessed($_   ) && overload::Method($_   , q[@{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[\@{}])/ },
+		constraint => sub {    ref($_   ) eq q[ARRAY] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[@{}])  },
+		inlined    => sub { qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\@{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -107,8 +175,8 @@ sub CodeLike ()
 	require Type::Tiny;
 	$cache{CodeLike} ||= "Type::Tiny"->new(
 		name       => "CodeLike",
-		constraint => sub {    ref($_   ) eq q[CODE] or Scalar::Util::blessed($_   ) && overload::Method($_   , q[&{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[\&{}])/ },
+		constraint => sub {    ref($_   ) eq q[CODE] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[&{}])  },
+		inlined    => sub { qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\&{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -121,14 +189,46 @@ sub TypeTiny ()
 		constraint => sub {  Scalar::Util::blessed($_   ) && $_   ->isa(q[Type::Tiny])  },
 		inlined    => sub { my $var = $_[1]; "Scalar::Util::blessed($var) && $var\->isa(q[Type::Tiny])" },
 		library    => __PACKAGE__,
+		_build_coercion => sub {
+			my $c = shift;
+			$c->add_type_coercions(_ForeignTypeConstraint(), \&to_TypeTiny);
+			$c->freeze;
+		},
+	);
+}
+
+sub _ForeignTypeConstraint ()
+{
+	require Type::Tiny;
+	$cache{_ForeignTypeConstraint} ||= "Type::Tiny"->new(
+		name       => "_ForeignTypeConstraint",
+		constraint => \&_is_ForeignTypeConstraint,
+		inlined    => sub { qq/ref($_[1]) && do { require Types::TypeTiny; Types::TypeTiny::_is_ForeignTypeConstraint($_[1]) }/ },
+		library    => __PACKAGE__,
 	);
 }
 
 my %ttt_cache;
 
+sub _is_ForeignTypeConstraint
+{
+	my $t = @_ ? $_[0] : $_;
+	return !!1 if ref $t eq 'CODE';
+	if (my $class = blessed $t)
+	{
+		return !!0 if $class->isa("Type::Tiny");
+		return !!1 if $class->isa("Moose::Meta::TypeConstraint");
+		return !!1 if $class->isa("MooseX::Types::TypeDecorator");
+		return !!1 if $class->isa("Validation::Class::Simple");
+		return !!1 if $class->isa("Validation::Class");
+		return !!1 if $t->can("check") && $t->can("get_message");
+	}
+	!!0;
+}
+
 sub to_TypeTiny
 {
-	my $t = $_[0];
+	my $t = @_ ? $_[0] : $_;
 	
 	return $t unless (my $ref = ref $t);
 	return $t if $ref =~ /^Type::Tiny\b/;
@@ -138,8 +238,10 @@ sub to_TypeTiny
 	if (my $class = blessed $t)
 	{
 		return $t                               if $class->isa("Type::Tiny");
+		return _TypeTinyFromMoose($t)           if $class eq "MooseX::Types::TypeDecorator";  # needed before MooseX::Types 0.35.
 		return _TypeTinyFromMoose($t)           if $class->isa("Moose::Meta::TypeConstraint");
 		return _TypeTinyFromMoose($t)           if $class->isa("MooseX::Types::TypeDecorator");
+		return _TypeTinyFromMouse($t)           if $class->isa("Mouse::Meta::TypeConstraint");
 		return _TypeTinyFromValidationClass($t) if $class->isa("Validation::Class::Simple");
 		return _TypeTinyFromValidationClass($t) if $class->isa("Validation::Class");
 		return _TypeTinyFromGeneric($t)         if $t->can("check") && $t->can("get_message"); # i.e. Type::API::Constraint
@@ -166,16 +268,22 @@ sub _TypeTinyFromMoose
 		return $ts if $ts->{_is_core};
 	}
 	
-	my %opts;
-	$opts{display_name} = $t->name;
-	$opts{constraint}   = $t->constraint;
-	$opts{parent}       = to_TypeTiny($t->parent)              if $t->has_parent;
-	$opts{inlined}      = sub { shift; $t->_inline_check(@_) } if $t->can("can_be_inlined") && $t->can_be_inlined;
-	$opts{message}      = sub { $t->get_message($_) }          if $t->has_message;
-	$opts{moose_type}   = $t;
+	my ($tt_class, $tt_opts) =
+		$t->can('parameterize')                          ? _TypeTinyFromMoose_parameterizable($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Enum')     ? _TypeTinyFromMoose_enum($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Class')    ? _TypeTinyFromMoose_class($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Role')     ? _TypeTinyFromMoose_role($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Union')    ? _TypeTinyFromMoose_union($t) :
+		$t->isa('Moose::Meta::TypeConstraint::DuckType') ? _TypeTinyFromMoose_ducktype($t) :
+		_TypeTinyFromMoose_baseclass($t);
 	
-	require Type::Tiny;
-	my $new = 'Type::Tiny'->new(%opts);
+	# Standard stuff to do with all type constraints from Moose,
+	# regardless of variety.
+	$tt_opts->{moose_type}   = $t;
+	$tt_opts->{display_name} = $t->name;
+	$tt_opts->{message}      = sub { $t->get_message($_) } if $t->has_message;
+	
+	my $new = $tt_class->new(%$tt_opts);
 	$ttt_cache{ refaddr($t) } = $new;
 	weaken($ttt_cache{ refaddr($t) });
 	
@@ -186,10 +294,75 @@ sub _TypeTinyFromMoose
 			moose_coercion  => $t->coercion,
 		);
 	} if $t->has_coercion;
-	
+		
 	return $new;
 }
 
+sub _TypeTinyFromMoose_baseclass
+{
+	my $t = shift;
+	my %opts;
+	$opts{parent}       = to_TypeTiny($t->parent)              if $t->has_parent;
+	$opts{constraint}   = $t->constraint;
+	$opts{inlined}      = sub { shift; $t->_inline_check(@_) } if $t->can("can_be_inlined") && $t->can_be_inlined;
+	
+	# Cowardly refuse to inline types that need to close over stuff
+	if ($opts{inlined}) {
+		my %env = %{ $t->inline_environment || {} };
+		delete($opts{inlined}) if keys %env;
+	}
+	
+	require Type::Tiny;
+	return 'Type::Tiny' => \%opts;
+}
+
+sub _TypeTinyFromMoose_union
+{
+	my $t = shift;
+	require Type::Tiny::Union;
+	return 'Type::Tiny::Union' => { type_constraints => [ map _TypeTinyFromMoose($_), @{$t->type_constraints} ] };
+}
+
+sub _TypeTinyFromMoose_enum
+{
+	my $t = shift;
+	require Type::Tiny::Enum;
+	return 'Type::Tiny::Enum' => { values => [@{ $t->values }] };
+}
+
+sub _TypeTinyFromMoose_class
+{
+	my $t = shift;
+	require Type::Tiny::Class;
+	return 'Type::Tiny::Class' => { class => $t->class };
+}
+
+sub _TypeTinyFromMoose_role
+{
+	my $t = shift;
+	require Type::Tiny::Role;
+	return 'Type::Tiny::Role' => { role => $t->role };
+}
+
+sub _TypeTinyFromMoose_ducktype
+{
+	my $t = shift;
+	require Type::Tiny::Duck;
+	return 'Type::Tiny::Duck' => { methods => [@{ $t->methods }] };
+}
+
+sub _TypeTinyFromMoose_parameterizable
+{
+	my $t = shift;
+	my ($class, $opts) = _TypeTinyFromMoose_baseclass($t);
+	$opts->{constraint_generator} = sub {
+		# convert args into Moose native types; not strictly necessary
+		my @args = map { TypeTiny->check($_) ? $_->moose_type : $_ } @_;
+		_TypeTinyFromMoose( $t->parameterize(@args) );
+	};
+	return ($class, $opts);
+}
+
 sub _TypeTinyFromValidationClass
 {
 	my $t = $_[0];
@@ -263,8 +436,6 @@ sub _TypeTinyFromGeneric
 {
 	my $t = $_[0];
 	
-	# XXX - handle inlining??
-	
 	my %opts = (
 		constraint => sub { $t->check(@_ ? @_ : $_) },
 		message    => sub { $t->get_message(@_ ? @_ : $_) },
@@ -275,6 +446,39 @@ sub _TypeTinyFromGeneric
 	$opts{coercion} = sub { $t->coerce(@_ ? @_ : $_) }
 		if $t->can("has_coercion") && $t->has_coercion && $t->can("coerce");
 	
+	if ($t->can('can_be_inlined') && $t->can_be_inlined && $t->can('inline_check')) {
+		$opts{inlined} = sub { $t->inline_check($_[1]) };
+	}
+	
+	require Type::Tiny;
+	my $new = "Type::Tiny"->new(%opts);
+	$ttt_cache{ refaddr($t) } = $new;
+	weaken($ttt_cache{ refaddr($t) });
+	return $new;
+}
+
+sub _TypeTinyFromMouse
+{
+	my $t = $_[0];
+	
+	my %opts = (
+		constraint => sub { $t->check(@_ ? @_ : $_) },
+		message    => sub { $t->get_message(@_ ? @_ : $_) },
+	);
+	
+	$opts{display_name} = $t->name if $t->can("name");
+	
+	$opts{coercion} = sub { $t->coerce(@_ ? @_ : $_) }
+		if $t->can("has_coercion") && $t->has_coercion && $t->can("coerce");
+
+	if ($t->{'constraint_generator'}) {
+		$opts{constraint_generator} = sub {
+			# convert args into Moose native types; not strictly necessary
+			my @args    = map { TypeTiny->check($_) ? $_->mouse_type : $_ } @_;
+			_TypeTinyFromMouse( $t->parameterize(@args) );
+		};
+	}
+
 	require Type::Tiny;
 	my $new = "Type::Tiny"->new(%opts);
 	$ttt_cache{ refaddr($t) } = $new;
@@ -375,6 +579,13 @@ Accepts coderefs and objects overloading codification.
 
 Accepts blessed L<Type::Tiny> objects.
 
+=item C<< _ForeignTypeConstraint >>
+
+Any reference which to_TypeTiny recognizes as something that can be coerced
+to a Type::Tiny object.
+
+Yeah, the underscore is included.
+
 =back
 
 =head2 Coercion Functions
@@ -441,7 +652,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
